<?php

require_once("Geocoder/Exception.php");
require_once("Geocoder/UnknownAddressException.php");

class PowerHour_Geocoder 
{
	const MAPS_HOST = "maps.google.com";
	
	/**
	 * Keine Fehler aufgetreten; die Adresse wurde erfolgreich analysiert. Der Geocode wurde zurckgegeben.
	 * @var int
	 */
	const G_GEO_SUCCESS = 200;

	/**
	 * Eine Routenanforderung konnte nicht erfolgreich analysiert werden. Die Anforderung kann 
	 * beispielsweise deshalb zurckgewiesen worden sein, weil sie mehr als die zulssige Anzahl 
	 * von Wegpunkten enthielt.
	 * @var int
	 */
	const G_GEO_BAD_REQUEST = 400;

	/**
	 * Eine Geokodierungs- oder Routenanforderung konnte nicht erfolgreich verarbeitet werden, da der 
	 * genaue Grund fr den Fehler nicht bekannt ist.
	 * @var int
	 */
	const G_GEO_SERVER_ERROR = 500;

	/**
	 * Der HTTP-Parameter q fehlt oder enthlt keinen Wert. Fr Geokodierungsanforderungen bedeutet dies, dass eine leere Adresse angegeben wurde. Fr Routenanforderungen bedeutet dies, dass keine Abfrage angegeben wurde.
	 * @var int
	 */
	const G_GEO_MISSING_QUERY = 601;

	/**
	 * Synonym fr G_GEO_MISSING_QUERY.
	 * @var int
	 */
	const G_GEO_MISSING_ADDRESS = 601;

	/**
	 * Es konnte keine entsprechende geografische Position fr die angegebene Adresse gefunden werden. 
	 * Dies kann daran liegen, dass die Adresse relativ neu oder mglicherweise falsch ist.
	 * @var int
	 */
	const G_GEO_UNKNOWN_ADDRESS = 602;

	/**
	 * Der Geocode fr die angegebene Adresse oder die Route fr die angegebene Richtungsanfrage kann 
	 * aus rechtlichen oder Vertragsgrnden nicht zurckgegeben werden.
	 * @var int
	 */
	const G_GEO_UNAVAILABLE_ADDRESS = 603;

	/**
	 * Das GDirections-Objekt konnte keinen Routenplan zwischen den Punkten in der Suchanfrage berechnen. 
	 * Dies ist blich, da keine Route zwischen den beiden Punkten oder keine Daten fr die Routenplanung 
	 * in dieser Region vorhanden sind.
	 * @var int
	 */
	const G_GEO_UNKNOWN_DIRECTIONS = 604;

	/**
	 * Der angegebene Schlssel ist entweder ungltig oder passt nicht zur Domain, fr die er 
	 * angegeben wurde.
	 * @var int
	 */
	const G_GEO_BAD_KEY = 610;

	/**
	 * Der angegebene Schlssel hat innerhalb des Zeitraums von 24 Stunden das Limit fr Anforderungen 
	 * berschritten oder zu viele Anforderungen in einem zu kurzen Zeitraum bermittelt. Wenn Sie 
	 * zahlreiche Anforderungen gleichzeitig oder kurz hintereinander bermitteln, verwenden Sie in 
	 * Ihrem Code einen Timer oder eine Pause, damit die Anforderungen nicht zu schnell bermittelt werden.
	 */
	const G_GEO_TOO_MANY_QUERIES = 620; 
	
	/**
	 * 	I M P O R T A N T
	 *  FILL IN YOUR GOOGLE MAPS KEY HERE
	 * @var string
	 */
	public static $GoogleMapsKey = "";
	
	protected $latitude = 0.0;
	protected $longitude = 0.0;
	protected $zip = "";
	
	function __construct() 
	{
	
	}
	
	
	/**
	 * @param $longitude the $longitude to set
	 */
	public function setLongitude($longitude) {
		$this->longitude = $longitude;
	}

	/**
	 * @return the $longitude
	 */
	public function getLongitude() {
		return $this->longitude;
	}

	/**
	 * @param $latitude the $latitude to set
	 */
	public function setLatitude($latitude) {
		$this->latitude = $latitude;
	}

	/**
	 * @return the $latitude
	 */
	public function getLatitude() {
		return $this->latitude;
	}
	/**
	 * @param $zip the $zip to set
	 */
	public function setZip($zip) {
		$this->zip = $zip;
	}

	/**
	 * @return the $zip
	 */
	public function getZip() {
		return $this->zip;
	}


	
	public function mapFromAddress($address)
	{		
		$address = $this->encodeUTF8($address);
		// Initialize delay in geocode speed
		$delay = 0;
		$base_url = "http://" . PowerHour_Geocoder::MAPS_HOST . "/maps/geo?output=xml" 
					."&key=" . PowerHour_Geocoder::$GoogleMapsKey;
		
		  $geocode_pending = true;
		
		  while ($geocode_pending) 
		  {
		    $request_url = $base_url . "&q=" . urlencode($address);
		    $xml = simplexml_load_file($request_url) or die("url not loading");
		
		    $status = intval($xml->Response->Status->code);
		    switch($status)
		    {
		    	case PowerHour_Geocoder::G_GEO_SUCCESS:
				      // Successful geocode
				      $geocode_pending = false;
				      $coordinates = $xml->Response->Placemark->Point->coordinates;
				      $coordinatesSplit = explode(",", $coordinates);
				      // Format: Longitude, Latitude, Altitude
				      $this->latitude = $coordinatesSplit[1];
				      $this->longitude = $coordinatesSplit[0];
				      //PLZ auslesen
				      $container = $xml->Response->Placemark->AddressDetails->Country->AdministrativeArea
				      					->SubAdministrativeArea->Locality;
			      	  if($container->DependentLocality)
			      	  {
			      	  	$container = $container->DependentLocality;	
			      	  }
				      $this->zip = $container->PostalCode->PostalCodeNumber;
				     break;
			     case PowerHour_Geocoder::G_GEO_TOO_MANY_QUERIES: 
				      // sent geocodes too fast
				      usleep(100000);
				      break;
			     case PowerHour_Geocoder::G_GEO_UNKNOWN_ADDRESS:
			     	$geocode_pending = false;
			     	throw new PowerHour_Geocoder_UnknownAddressException("Address: ".$address." is unknown.");
			     	break;
			     default:
				     	// failure to geocode
				      	$geocode_pending = false;
				      	throw new PowerHour_Geocoder_Exception(	"Address " . $address . " failed to geocode. ".
				      											"Received status " . $status . "<br />");
				      break;
		    } 
		  }
	} 
	
	/**
	 * Codiert einen String nur dann als utf8, wenn dieser nicht bereits codiert wurde
	 * @param string $in_str
	 */
	protected function encodeUTF8($in_str)
	{
	  $cur_encoding = mb_detect_encoding($in_str) ;
	  if($cur_encoding == "UTF-8" && mb_check_encoding($in_str,"UTF-8"))
	    return $in_str;
	  else
	    return utf8_encode($in_str);
	}
	
}

?>